@@ -347,34 +347,26 @@ class Variable(Reprable, metaclass=VariableMeta):
         return var
 
     def __eq__(self, other):
-        # pylint: disable=protected-access,import-outside-toplevel
+        if type(self) is not type(other):
+            return False
 
-        def to_match(var):
-            if var._compute_value is None:
-                return var
-            elif isinstance(var._compute_value, Identity):
-                return var._compute_value.variable
-            return None
-
-        from Orange.preprocess.transformation import Identity
-        return type(self) is type(other) and (
-            self.name == other.name
-            and self._compute_value == other._compute_value
-            or
-            (self.compute_value or other.compute_value)
-            and to_match(self) == to_match(other) != None)
+        var1 = self._get_identical_source(self)
+        var2 = self._get_identical_source(other)
+        # pylint: disable=protected-access
+        return var1.name == var2.name \
+               and var1._compute_value == var2._compute_value
 
     def __hash__(self):
-        # Two variables that are not equal can have the same hash.
-        # This happens if one has compute_value == Identity and the other
-        # doesn't have compute_value, or they have a different Identity.
-        # Having the same hash while not being equal is of course allowed.
-        # pylint: disable=import-outside-toplevel
+        var = self._get_identical_source(self)
+        return hash((var.name, type(self), var._compute_value))
+
+    @staticmethod
+    def _get_identical_source(var):
+        # pylint: disable=protected-access,import-outside-toplevel
         from Orange.preprocess.transformation import Identity
-        compute_value = self._compute_value
-        if isinstance(self._compute_value, Identity):
-            compute_value = None
-        return hash((self.name, type(self), compute_value))
+        while isinstance(var._compute_value, Identity):
+            var = var._compute_value.variable
+        return var
 
     @classmethod
     def make(cls, name, *args, **kwargs):
@@ -482,9 +474,13 @@ class Variable(Reprable, metaclass=VariableMeta):
         # Use make to unpickle variables.
         return make_variable, (self.__class__, self._compute_value, self.name), self.__dict__
 
-    def copy(self, compute_value=None, *, name=None, **kwargs):
+    _CopyComputeValue = object()
+
+    def copy(self, compute_value=_CopyComputeValue, *, name=None, **kwargs):
+        if compute_value is self._CopyComputeValue:
+            compute_value = self.compute_value
         var = type(self)(name=name or self.name,
-                         compute_value=compute_value or self.compute_value,
+                         compute_value=compute_value,
                          sparse=self.sparse, **kwargs)
         var.attributes = dict(self.attributes)
         return var
@@ -590,7 +586,8 @@ class ContinuousVariable(Variable):
 
     str_val = repr_val
 
-    def copy(self, compute_value=None, *, name=None, **kwargs):
+    def copy(self, compute_value=Variable._CopyComputeValue,
+             *, name=None, **kwargs):
         # pylint understand not that `var` is `DiscreteVariable`:
         # pylint: disable=protected-access
         number_of_decimals = kwargs.pop("number_of_decimals", None)
@@ -826,7 +823,8 @@ class DiscreteVariable(Variable):
                                self.values, self.ordered), \
             __dict__
 
-    def copy(self, compute_value=None, *, name=None, values=None, **_):
+    def copy(self, compute_value=Variable._CopyComputeValue,
+             *, name=None, values=None, **_):
         # pylint: disable=arguments-differ
         if values is not None and len(values) != len(self.values):
             raise ValueError(
@@ -956,7 +954,7 @@ class TimeVariable(ContinuousVariable):
         self.have_date = have_date
         self.have_time = have_time
 
-    def copy(self, compute_value=None, *, name=None, **_):
+    def copy(self, compute_value=Variable._CopyComputeValue, *, name=None, **_):
         return super().copy(compute_value=compute_value, name=name,
                             have_date=self.have_date, have_time=self.have_time)
 
