@@ -9,7 +9,8 @@ from AnyQt.QtCore import Qt
 from AnyQt.QtTest import QTest
 import baycomp
 
-from Orange.classification import MajorityLearner, LogisticRegressionLearner
+from Orange.classification import MajorityLearner, LogisticRegressionLearner, \
+    RandomForestLearner
 from Orange.classification.majority import ConstantModel
 from Orange.data import Table, Domain, DiscreteVariable, ContinuousVariable
 from Orange.evaluation import Results, TestOnTestData, scoring
@@ -24,7 +25,9 @@ from Orange.widgets.evaluate.utils import BUILTIN_SCORERS_ORDER
 from Orange.widgets.settings import (
     ClassValuesContextHandler, PerfectDomainContextHandler)
 from Orange.widgets.tests.base import WidgetTest
-from Orange.widgets.tests.utils import simulate
+from Orange.widgets.tests.utils import simulate, possible_duplicate_table
+from Orange.widgets.utils.state_summary import (format_summary_details,
+                                                format_multiple_summaries)
 from Orange.tests import test_filename
 
 
@@ -166,25 +169,38 @@ class TestOWTestAndScore(WidgetTest):
                 "yyyy"))
         )
         self.widget.n_folds = 0
-        self.assertFalse(self.widget.Error.only_one_class_var_value.is_shown())
+        self.assertFalse(self.widget.Error.train_data_error.is_shown())
         self.send_signal("Data", table)
         self.send_signal("Learner", MajorityLearner(), 0, wait=1000)
-        self.assertTrue(self.widget.Error.only_one_class_var_value.is_shown())
+        self.assertTrue(self.widget.Error.train_data_error.is_shown())
 
-    def test_nan_class(self):
-        """
-        Do not crash on a data with only nan class values.
-        GH-2751
-        """
-        def assertErrorShown(data, is_shown):
+    def test_data_errors(self):
+        """ Test all data_errors """
+        def assertErrorShown(data, is_shown, message):
             self.send_signal("Data", data)
-            self.assertEqual(is_shown, self.widget.Error.no_class_values.is_shown())
+            self.assertEqual(is_shown, self.widget.Error.train_data_error.is_shown())
+            self.assertEqual(message, str(self.widget.Error.train_data_error))
 
         data = Table("iris")[::30]
         data.Y[:] = np.nan
 
-        for data, is_shown in zip([None, data, Table("iris")[:30]], [False, True, False]):
-            assertErrorShown(data, is_shown)
+        iris_empty_x = Table.from_table(
+            Domain([], data.domain.class_var), Table("iris")
+        )
+
+        for data, is_shown, message in zip(
+                [None, data, Table("iris")[:30], iris_empty_x, data[:0]],
+                [False, True, True, True, True],
+                [
+                    "",
+                    "Target variable has no values.",
+                    "Target variable has only one value.",
+                    "Data has no features to learn from.",
+                    "Train dataset is empty."
+                ]
+
+        ):
+            assertErrorShown(data, is_shown, message)
 
     def test_addon_scorers(self):
         try:
@@ -319,15 +335,19 @@ class TestOWTestAndScore(WidgetTest):
         self.send_signal(self.widget.Inputs.learner, learner, 0, wait=5000)
         return self._retrieve_scores()
 
-    def test_scores_constant_all_same(self):
+    def test_scores_constant(self):
         table = Table.from_list(
             self.scores_domain,
-            list(zip(*self.scores_table_values + [list("yyyy")]))
+            list(zip(*self.scores_table_values + [list("yyyn")]))
         )
 
-        self.assertTupleEqual(self._test_scores(
-            table, table, ConstantLearner(), OWTestAndScore.TestOnTest, None),
-                              (None, 1, 1, 1, 1))
+        self.assertTupleEqual(
+            self._test_scores(
+                table, table[:3], ConstantLearner(),
+                OWTestAndScore.TestOnTest, None
+            ),
+            (None, 1, 1, 1, 1)
+        )
 
     def test_scores_log_reg_overfitted(self):
         table = Table.from_list(
@@ -362,26 +382,33 @@ class TestOWTestAndScore(WidgetTest):
         table_test = Table.from_list(
             self.scores_domain,
             list(zip(*(self.scores_table_values + [list("yynn")]))))
-        self.assertTupleEqual(self._test_scores(
-            table_train, table_test, LogisticRegressionLearner(),
-            OWTestAndScore.TestOnTest, None),
-                              (0, 0, 0, 0, 0))
+
+        lr = LogisticRegressionLearner()
+        np.testing.assert_almost_equal(
+            self._test_scores(
+                table_train, table_test, lr, OWTestAndScore.TestOnTest, None
+            ),
+            (0, 0.25, 0.2, 0.1666666, 0.25),
+        )
 
     def test_scores_log_reg_advanced(self):
         table_train = Table.from_list(
-            self.scores_domain, list(zip(
-                [1, 1, 1.23, 23.8, 5.], [1., 2., 3., 4., 3.], "yyynn"))
+            self.scores_domain,
+            list(zip([1, 1, 1.23, 23.8, 5.], [1., 2., 3., 4., 3.], "yyynn"))
         )
         table_test = Table.from_list(
-            self.scores_domain, list(zip(
-                [1, 1, 1.23, 23.8, 5.], [1., 2., 3., 4., 3.], "yynnn"))
+            self.scores_domain,
+            list(zip([1, 1, 1.23, 23.8, 5.], [1., 2., 3., 4., 3.], "yynnn"))
         )
 
+        lr = LogisticRegressionLearner()
+        np.testing.assert_
         np.testing.assert_almost_equal(
-            self._test_scores(table_train, table_test,
-                              LogisticRegressionLearner(),
-                              OWTestAndScore.TestOnTest, None),
-            (2 / 3, 0.8, 0.8, 13 / 15, 0.8))
+            self._test_scores(
+                table_train, table_test, lr, OWTestAndScore.TestOnTest, None
+            ),
+            (1, 0.8, 0.8, 13 / 15, 0.8)
+        )
 
     def test_scores_cross_validation(self):
         """
@@ -619,6 +646,53 @@ class TestOWTestAndScore(WidgetTest):
                 label = w.comparison_table.cellWidget(1, 0)
                 self.assertEqual(label.text(), "NA")
 
+    def test_summary(self):
+        """Check if the status bar updates when data is on input"""
+        iris = Table("iris")
+        train, test = iris[:120], iris[120:]
+        info = self.widget.info
+        no_input, no_output = "No data on input", "No data on output"
+
+        self.send_signal(self.widget.Inputs.train_data, train)
+        self.send_signal(self.widget.Inputs.learner, LogisticRegressionLearner(), 0)
+        summary, details = f"{len(train)}", format_summary_details(train)
+        self.assertEqual(info._StateInfo__input_summary.brief, summary)
+        self.assertEqual(info._StateInfo__input_summary.details, details)
+        output = self.get_output(self.widget.Outputs.predictions)
+        summary, details = f"{len(output)}", format_summary_details(output)
+        self.assertEqual(info._StateInfo__output_summary.brief, summary)
+        self.assertEqual(info._StateInfo__output_summary.details, details)
+
+        self.send_signal(self.widget.Inputs.test_data, test)
+        summary = f"{len(train)}, {len(test)}"
+        details = format_multiple_summaries([("Data", train), ("Test data", test)])
+        self.assertEqual(info._StateInfo__input_summary.brief, summary)
+        self.assertEqual(info._StateInfo__input_summary.details, details)
+        output = self.get_output(self.widget.Outputs.predictions)
+        summary, details = f"{len(output)}", format_summary_details(output)
+        self.assertEqual(info._StateInfo__output_summary.brief, summary)
+        self.assertEqual(info._StateInfo__output_summary.details, details)
+
+        self.send_signal(self.widget.Inputs.train_data, None)
+        summary, details = f"{len(test)}", format_summary_details(test)
+        self.assertEqual(info._StateInfo__input_summary.brief, summary)
+        self.assertEqual(info._StateInfo__input_summary.details, details)
+        self.assertEqual(info._StateInfo__output_summary.brief, "")
+        self.assertEqual(info._StateInfo__output_summary.details, no_output)
+
+        self.send_signal(self.widget.Inputs.test_data, None)
+        self.assertEqual(info._StateInfo__input_summary.brief, "")
+        self.assertEqual(info._StateInfo__input_summary.details, no_input)
+        self.assertEqual(info._StateInfo__output_summary.brief, "")
+        self.assertEqual(info._StateInfo__output_summary.details, no_output)
+
+    def test_unique_output_domain(self):
+        data = possible_duplicate_table('random forest')
+        self.send_signal(self.widget.Inputs.train_data, data)
+        self.send_signal(self.widget.Inputs.learner, RandomForestLearner(), 0)
+        output = self.get_output(self.widget.Outputs.predictions)
+        self.assertEqual(output.domain.metas[0].name, 'random forest (1)')
+
 
 class TestHelpers(unittest.TestCase):
     def test_results_one_vs_rest(self):
