@@ -27,6 +27,7 @@ from Orange.widgets.utils.annotated_data import (create_annotated_table,
                                                  ANNOTATED_DATA_SIGNAL_NAME)
 from Orange.widgets.utils.itemmodels import DomainModel
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.visualize.utils import (
     CanvasText, CanvasRectangle, ViewWithPress, VizRankDialog)
 from Orange.widgets.visualize.utils.plotutils import wrap_legend_items
@@ -332,6 +333,9 @@ class OWMosaicDisplay(OWWidget):
         self.canvas_view.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         self.canvas_view.setRenderHint(QPainter.Antialiasing)
 
+        self.info.set_input_summary(self.info.NoInput)
+        self.info.set_output_summary(self.info.NoOutput)
+
         box = gui.vBox(self.controlArea, box=True)
         self.model_1 = DomainModel(
             order=DomainModel.MIXED, valid_types=DomainModel.PRIMITIVE)
@@ -442,8 +446,10 @@ class OWMosaicDisplay(OWWidget):
         if self.data is None:
             self.discrete_data = None
             self.init_combos(None)
+            self.info.set_input_summary(self.info.NoInput)
             return
 
+        self.info.set_input_summary(len(data),format_summary_details(data))
         self.init_combos(self.data)
         self.openContext(self.data)
 
@@ -518,6 +524,7 @@ class OWMosaicDisplay(OWWidget):
             self.Outputs.selected_data.send(None)
             self.Outputs.annotated_data.send(
                 create_annotated_table(self.data, []))
+            self.info.set_output_summary(self.info.NoOutput)
             return
         filters = []
         self.Warning.no_cont_selection_sql.clear()
@@ -539,6 +546,10 @@ class OWMosaicDisplay(OWWidget):
         sel_idx = [i for i, id in enumerate(self.data.ids) if id in idset]
         if self.discrete_data is not self.data:
             selection = self.data[sel_idx]
+
+        summary = len(selection) if selection else self.info.NoOutput
+        details = format_summary_details(selection) if selection else ""
+        self.info.set_output_summary(summary, details)
         self.Outputs.selected_data.send(selection)
         self.Outputs.annotated_data.send(
             create_annotated_table(self.data, sel_idx))
@@ -994,5 +1005,5 @@ def get_conditional_distribution(data, attrs):
 
 
 if __name__ == "__main__":  # pragma: no cover
-    data = Table("zoo")
-    WidgetPreview(OWMosaicDisplay).run(data, set_subset_data=data[::10])
+    dataset = Table("zoo")
+    WidgetPreview(OWMosaicDisplay).run(dataset, set_subset_data=dataset[::10])
