@@ -32,6 +32,7 @@ from Orange.widgets.utils.annotated_data import (create_annotated_table,
                                                  ANNOTATED_DATA_SIGNAL_NAME)
 from Orange.widgets.utils.sql import check_sql_input
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details, format_multiple_summaries
 from Orange.widgets.widget import Input, Output, Msg
 
 
@@ -112,6 +113,9 @@ class OWVennDiagram(widget.OWWidget):
         self.vennwidget.itemTextEdited.connect(self._on_itemTextEdited)
         self.scene.selectionChanged.connect(self._on_selectionChanged)
 
+        self.info.set_input_summary(self.info.NoInput)
+        self.info.set_output_summary(self.info.NoOutput)
+
         self.scene.addItem(self.vennwidget)
 
         controls = gui.hBox(self.mainArea)
@@ -198,6 +202,7 @@ class OWVennDiagram(widget.OWWidget):
 
     def handleNewSignals(self):
         self._inputUpdate = False
+        self.set_input_summary()
         self.vennwidget.clear()
         if not self.settings_compatible():
             self.invalidateOutput()
@@ -213,6 +218,19 @@ class OWVennDiagram(widget.OWWidget):
         self._updateInfo()
         super().handleNewSignals()
 
+    def set_input_summary(self):
+        names = [self.data[k].name for k in self.data.keys()]
+        tables = [self.data[k].table for k in self.data.keys()]
+        n_data = [self.info.format_number(len(self.data[k].table)) for k in
+                  self.data.keys()]
+
+        summary, details, kwargs = self.info.NoInput, "", {}
+        if self.data:
+            summary = ", ".join(n_data)
+            details = format_multiple_summaries(zip(names, tables))
+            kwargs = {"format": Qt.RichText}
+        self.info.set_input_summary(summary, details, **kwargs)
+
     def intersectionStringAttrs(self):
         sets = [set(string_attributes(data_.table.domain)) for data_ in self.data.values()]
         if sets:
@@ -366,10 +384,12 @@ class OWVennDiagram(widget.OWWidget):
     def merge_data(self, domain, values, ids=None):
         X, metas, class_vars = None, None, None
         renamed = []
+        names = [var.name for val in domain.values() for var in val]
+        unique_names = iter(get_unique_names_duplicates(names))
+        
         for val in domain.values():
-            names = [var.name for var in val]
-            unique_names = get_unique_names_duplicates(names)
-            for n, u, idx, var in zip(names, unique_names, count(), val):
+            for n, idx, var in zip(names, count(), val):
+                u = next(unique_names)
                 if n != u:
                     val[idx] = var.copy(name=u)
                     renamed.append(n)
@@ -654,6 +674,7 @@ class OWVennDiagram(widget.OWWidget):
         if not self.vennwidget.vennareas() or not self.data:
             self.Outputs.selected_data.send(None)
             self.Outputs.annotated_data.send(None)
+            self.info.set_output_summary(self.info.NoOutput)
             return
 
         self.selected_items = reduce(
@@ -677,6 +698,9 @@ class OWVennDiagram(widget.OWWidget):
             if self.selected_items:
                 selected = self.create_from_columns(self.selected_items, selected_keys, True)
 
+        summary = len(selected) if selected else self.info.NoOutput
+        details = format_summary_details(selected) if selected else ""
+        self.info.set_output_summary(summary, details)
         self.Outputs.selected_data.send(selected)
         self.Outputs.annotated_data.send(annotated)
 
